/* Metrics.java
 * =========================================================================
 * This file is originally part of the JMathTeX Library - http://jmathtex.sourceforge.net
 * 
 * Copyright (C) 2004-2007 Universiteit Gent
 * Copyright (C) 2009 DENIZET Calixte
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 * 
 */

package org.scilab.forge.jlatexmath;

/**
 * Contains the metrics for 1 character: width, height, depth and italic correction.
 */
public class Metrics {

    private final float w;
    private final float h;
    private final float d;
    private final float i;
    private final float s;
    
    public Metrics(float w, float h, float d, float i, float factor, float size) {
	this.w = w * factor;
	this.h = h * factor;
	this.d = d * factor;
	this.i = i * factor;
	this.s = size;
    }

    public float getWidth() {
	return w;
    }

    public float getHeight() {
	return h;
    }

    public float getDepth() {
	return d;
    }

    public float getItalic() {
	return i;
    }
    
    public float getSize() {
	return s;
    }
}
